@@ -684,8 +684,6 @@ gvfs_udisks2_volume_new (GVfsUDisks2VolumeMonitor   *monitor,
   volume->activation_root = activation_root != NULL ? g_object_ref (activation_root) : NULL;
 
   volume->drive = drive;
-  if (drive != NULL)
-    gvfs_udisks2_drive_set_volume (drive, volume);
 
   update_volume (volume);
 
@@ -733,7 +731,6 @@ gvfs_udisks2_volume_set_mount (GVfsUDisks2Volume *volume,
 
       volume->mount = mount;
       update_volume (volume);
-      emit_changed (volume);
     }
 }
 
@@ -745,7 +742,6 @@ gvfs_udisks2_volume_unset_mount (GVfsUDisks2Volume *volume,
     {
       volume->mount = NULL;
       update_volume (volume);
-      emit_changed (volume);
     }
 }
 
@@ -758,7 +754,6 @@ gvfs_udisks2_volume_set_drive (GVfsUDisks2Volume *volume,
       if (volume->drive != NULL)
         gvfs_udisks2_drive_unset_volume (volume->drive, volume);
       volume->drive = drive;
-      emit_changed (volume);
     }
 }
 
@@ -769,7 +764,6 @@ gvfs_udisks2_volume_unset_drive (GVfsUDisks2Volume *volume,
   if (volume->drive == drive)
     {
       volume->drive = NULL;
-      emit_changed (volume);
     }
 }
 
@@ -1853,6 +1847,14 @@ gvfs_udisks2_volume_get_block (GVfsUDisks2Volume *volume)
   return volume->block;
 }
 
+GVfsUDisks2Drive *
+gvfs_udisks2_volume_get_udisks2_drive (GVfsUDisks2Volume *volume)
+{
+  g_return_val_if_fail (GVFS_IS_UDISKS2_VOLUME (volume), NULL);
+
+  return volume->drive;
+}
+
 GUnixMountPoint *
 gvfs_udisks2_volume_get_mount_point (GVfsUDisks2Volume *volume)
 {
